# Generated by Django 3.0.5 on 2022-11-18 08:35

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('drf_rbac', '0002_auto_20221118_0826'),
    ]

    operations = [
        migrations.AlterField(
            model_name='role',
            name='user',
            field=models.ManyToManyField(blank=True, through='drf_rbac.UserRoleMembership', to=settings.AUTH_USER_MODEL, verbose_name='Related User-Role'),
        ),
    ]
